@@ -26,7 +26,8 @@ class ScatterChartData(object):
 	             filter_unigrams=False,
 	             term_ranker=AbsoluteFrequencyRanker,
 	             use_non_text_features=False,
-	             term_significance=None):
+	             term_significance=None,
+	             terms_to_include=None):
 		'''
 
 		Parameters
@@ -52,8 +53,10 @@ class ScatterChartData(object):
 			TermRanker class for determining term frequency ranks.
 		use_non_text_features : bool, default = False
 			Use non-BoW features (e.g., Empath) instead of text features
-		term_significance : TermSignifiance instance or None
-			Way of getting signfiance scores.  If None, p values will not be added.
+		term_significance : TermSignificance instance or None
+			Way of getting significance scores.  If None, p values will not be added.
+		terms_to_include : set or None
+			Only annotate these terms in chart
 		'''
 		self.jitter = jitter
 		self.minimum_term_frequency = minimum_term_frequency
@@ -65,6 +68,7 @@ class ScatterChartData(object):
 		self.max_terms = max_terms
 		self.use_non_text_features = use_non_text_features
 		self.term_significance = term_significance
+		self.terms_to_include = terms_to_include
 		np.random.seed(seed)
 
 
@@ -75,20 +79,47 @@ class ScatterChart:
 	def __init__(self,
 	             term_doc_matrix,
 	             **kwargs):
+		'''
+		Parameters
+		----------
+		term_doc_matrix: term document matrix to create chart from
+
+		Remaining parameters are from ScatterChartData
+		'''
 		self.term_doc_matrix = term_doc_matrix
 		self.scatterchartdata = ScatterChartData(**kwargs)
 		self.x_coords = None
 		self.y_coords = None
+		self.original_x = None
+		self.original_y = None
+		self._rescale_x = None
+		self._rescale_y = None
+		self.used = False
 
-	def inject_coordinates(self, x_coords, y_coords):
+	def inject_coordinates(self,
+	                       x_coords,
+	                       y_coords,
+	                       rescale_x=None,
+	                       rescale_y=None,
+	                       original_x=None,
+	                       original_y=None):
 		'''
 		Inject custom x and y coordinates for each term into chart.
 
 		Parameters
 		----------
-		x_coords: array-like, positions on x-axis \in [0,1]
-		y_coords: array-like, positions on y-axis \in [0,1]
-
+		x_coords: array-like
+			positions on x-axis \in [0,1]
+		y_coords: array-like
+			positions on y-axis \in [0,1]
+		rescale_x: lambda list[0,1]: list[0,1], default identity
+			Rescales x-axis after filtering
+		rescale_y: lambda list[0,1]: list[0,1], default identity
+			Rescales y-axis after filtering
+		original_x : array-like, optional
+			Original, unscaled x-values.  Defaults to x_coords
+		original_y : array-like, optional
+			Original, unscaled y-values.  Defaults to y_coords
 		Returns
 		-------
 		self: ScatterChart
@@ -98,6 +129,10 @@ class ScatterChart:
 		self._verify_coordinates(y_coords, 'y')
 		self.x_coords = x_coords
 		self.y_coords = y_coords
+		self._rescale_x = rescale_x
+		self._rescale_y = rescale_y
+		self.original_x = x_coords if original_x is None else original_x
+		self.original_y = y_coords if original_y is None else original_y
 
 	def _verify_coordinates(self, coords, name):
 		if len(coords) != self.term_doc_matrix.get_num_terms():
@@ -113,7 +148,11 @@ class ScatterChart:
 	            category_name=None,
 	            not_category_name=None,
 	            scores=None,
-	            transform=percentile_alphabetical):
+	            transform=percentile_alphabetical,
+	            title_case_names=False,
+	            not_categories=None,
+	            neutral_categories=None,
+	            extra_categories=None):
 		'''
 
 		Parameters
@@ -128,6 +167,14 @@ class ScatterChart:
 			Scores to use for coloring.  Defaults to None, or np.array(self.term_doc_matrix.get_scaled_f_scores(category))
 		transform : function, optional
 			Function for ranking terms.  Defaults to scattertext.Scalers.percentile_lexicographic.
+		title_case_names : bool, default False
+		  Title case category name and no-category name?
+		not_categories : list, optional
+			List of categories to use as "not category".  Defaults to all others.
+		neutral_categories : list, optional
+			List of categories to use as neutral.  Defaults [].
+		extra_categories : list, optional
+			List of categories to use as extra.  Defaults [].
 
 		Returns
 		-------
@@ -138,6 +185,8 @@ class ScatterChart:
 		  data: [{term:,
 		          x:frequency [0-1],
 		          y:frequency [0-1],
+		          ox: score,
+		          oy: score,
               s: score,
               os: original score,
               p: p-val,
@@ -149,14 +198,99 @@ class ScatterChart:
               ncat25k: freq per 25k in non-category}, ...]}}
 
 		'''
-		all_categories, other_categories = self._get_category_names(category)
-		df = self._term_rank_score_and_frequency_df(all_categories, category, scores)
+		if self.used:
+			raise Exception("Cannot reuse a ScatterChart constructor")
+
+		all_categories = self.term_doc_matrix.get_categories()
+		assert category in all_categories
+
+		if not_categories is None:
+			not_categories = [c for c in all_categories if c != category]
+			neutral_categories = []
+			extra_categories = []
+		elif neutral_categories is None:
+			neutral_categories = [c for c in all_categories
+			                      if c not in [category] + not_categories]
+			extra_categories = []
+		elif extra_categories is None:
+			extra_categories = [c for c in all_categories
+			                    if c not in [category] + not_categories + neutral_categories]
+		all_categories = [category] + not_categories + neutral_categories + extra_categories
+
+		term_ranker = self.scatterchartdata.term_ranker(self.term_doc_matrix)
+		if self.scatterchartdata.use_non_text_features:
+			term_ranker.use_non_text_features()
+
+		df = term_ranker.get_ranks()
+		if self.x_coords is not None:
+			df['x'] = self.x_coords
+			df['y'] = self.y_coords
+
+		if not self.original_x is None:
+			try:
+				df['ox'] = self.original_x.values
+			except AttributeError:
+				df['ox'] = self.original_x
+
+		if not self.original_y is None:
+			try:
+				df['oy'] = self.original_y.values
+			except AttributeError:
+				df['oy'] = self.original_y
+
+		if scores is None:
+			scores = self._get_default_scores(category, not_categories, df)
+		# np.array(self.term_doc_matrix.get_rudder_scores(category))
+		# df['category score'] = np.array(self.term_doc_matrix.get_rudder_scores(category))
+		category_column_name = category + ' freq'
+		df['category score'] = CornerScore.get_scores_for_category(
+			df[category_column_name],
+			df[[c+' freq' for c in not_categories]].sum(axis=1)
+		)
+		if self.scatterchartdata.term_significance is not None:
+			df['p'] = get_p_vals(df, category_column_name,
+			                     self.scatterchartdata.term_significance)
+		df['not category score'] = CornerScore.get_scores_for_category(
+			df[[c+' freq' for c in not_categories]].sum(axis=1),
+			df[category_column_name]
+		)
+		df['color_scores'] = scores
+		if self.scatterchartdata.terms_to_include is None:
+			df = self._filter_bigrams_by_minimum_not_category_term_freq(
+				category_column_name, not_categories, df)
+			df = filter_bigrams_by_pmis(
+				self._filter_by_minimum_term_frequency(all_categories, df),
+				threshold_coef=self.scatterchartdata.pmi_threshold_coefficient
+			)
+
+		if self.scatterchartdata.filter_unigrams:
+			df = filter_out_unigrams_that_only_occur_in_one_bigram(df)
+		if len(df) == 0:
+			raise NoWordMeetsTermFrequencyRequirementsError()
+		df['category score rank'] = rankdata(df['category score'], method='ordinal')
+		df['not category score rank'] = rankdata(df['not category score'], method='ordinal')
+		if self.scatterchartdata.max_terms and self.scatterchartdata.max_terms < len(df):
+			assert self.scatterchartdata.max_terms > 0
+			df = self._limit_max_terms(category, df)
+		df = df.reset_index()
+
+
+
 		if self.x_coords is None:
 			self.x_coords, self.y_coords = self._get_coordinates_from_transform_and_jitter_frequencies \
-				(category, df, other_categories, transform)
+				(category, df, not_categories, transform)
 			df['x'], df['y'] = self.x_coords, self.y_coords
-		df['not cat freq'] = df[[x for x in other_categories]].sum(axis=1)
-		json_df = df[['x', 'y', 'term']]
+			df['ox'], df['oy'] = self.x_coords, self.y_coords
+
+		df['not cat freq'] = df[[x + ' freq' for x in not_categories]].sum(axis=1)
+		if neutral_categories != []:
+			df['neut cat freq'] = df[[x + ' freq' for x in neutral_categories]].sum(axis=1).fillna(0)
+		if extra_categories != []:
+			df['extra cat freq'] = df[[x + ' freq' for x in extra_categories]].sum(axis=1).fillna(0)
+
+
+		json_df = df[['x', 'y', 'ox', 'oy', 'term']]
+
 		if self.scatterchartdata.term_significance:
 			json_df['p'] = df['p']
 		self._add_term_freq_to_json_df(json_df, df, category)
@@ -165,6 +299,12 @@ class ScatterChart:
 		if not self.scatterchartdata.use_non_text_features:
 			json_df['bg'] = self._get_corpus_characteristic_scores(json_df)
 
+		self._preform_axis_rescale(json_df, self._rescale_x, 'x')
+		self._preform_axis_rescale(json_df, self._rescale_y, 'y')
+
+		if self.scatterchartdata.terms_to_include is not None:
+			json_df = self._use_only_selected_terms(json_df)
+
 		category_terms = list(json_df.sort_values('s')['term'][:10])
 		not_category_terms = list(json_df.sort_values('s')['term'][:10])
 		if category_name is None:
@@ -173,16 +313,32 @@ class ScatterChart:
 			not_category_name = 'Not ' + category_name
 
 		def better_title(x):
-			return ' '.join([t[0].upper() + t[1:].lower() for t in x.split()])
+			if title_case_names:
+				return ' '.join([t[0].upper() + t[1:].lower() for t in x.split()])
+			else:
+				return x
 
 		j = {'info': {'category_name': better_title(category_name),
 		              'not_category_name': better_title(not_category_name),
 		              'category_terms': category_terms,
 		              'not_category_terms': not_category_terms,
-		              'category_internal_name': category}}
+		              'category_internal_name': category,
+		              'not_category_internal_names': not_categories,
+		              'categories': self.term_doc_matrix.get_categories(),
+		              'neutral_category_internal_names': neutral_categories,
+		              'extra_category_internal_names': extra_categories}}
 		j['data'] = json_df.sort_values(by=['x', 'y', 'term']).to_dict(orient='records')
 		return j
 
+	def _use_only_selected_terms(self, json_df):
+		term_df = pd.DataFrame({"term": self.scatterchartdata.terms_to_include})
+		return pd.merge(json_df, term_df, on='term', how='inner')
+
+	def _preform_axis_rescale(self, json_df, rescaler, variable_to_rescale):
+		if rescaler is not None:
+			json_df[variable_to_rescale] = rescaler(json_df[variable_to_rescale])
+			assert json_df[variable_to_rescale].min() >= 0 and json_df[variable_to_rescale].max() <= 1
+
 	def _get_corpus_characteristic_scores(self, json_df):
 		bg_terms = self.term_doc_matrix.get_scaled_f_scores_vs_background()
 		bg_terms = bg_terms['Scaled f-score']
@@ -195,10 +351,26 @@ class ScatterChart:
 	def _add_term_freq_to_json_df(self, json_df, term_freq_df, category):
 		json_df['cat25k'] = (((term_freq_df[category + ' freq'] * 1.
 		                       / term_freq_df[category + ' freq'].sum()) * 25000)
-		                     .apply(np.round).astype(np.int))
+			.apply(np.round).astype(np.int))
 		json_df['ncat25k'] = (((term_freq_df['not cat freq'] * 1.
 		                        / term_freq_df['not cat freq'].sum()) * 25000)
-		                      .apply(np.round).astype(np.int))
+			.apply(np.round).astype(np.int))
+		if 'neut cat freq' in term_freq_df:
+			json_df['neut25k'] = (((term_freq_df['neut cat freq'] * 1.
+			                        / term_freq_df['neut cat freq'].sum()) * 25000)
+				.apply(np.round).astype(np.int))
+			json_df['neut'] = term_freq_df['neut cat freq']
+		else:
+			json_df['neut25k'] = 0
+			json_df['neut'] = 0
+		if 'extra cat freq' in term_freq_df:
+			json_df['extra25k'] = (((term_freq_df['extra cat freq'] * 1.
+			                         / term_freq_df['extra cat freq'].sum()) * 25000)
+				.apply(np.round).astype(np.int))
+			json_df['extra'] = term_freq_df['extra cat freq']
+		else:
+			json_df['extra25k'] = 0
+			json_df['extra'] = 0
 
 	def _get_category_names(self, category):
 		other_categories = [val + ' freq' for _, val \
@@ -212,7 +384,7 @@ class ScatterChart:
 	                                                           df,
 	                                                           other_categories,
 	                                                           transform):
-		not_counts = df[other_categories].sum(axis=1)
+		not_counts = df[[c + ' freq'  for c in other_categories]].sum(axis=1)
 		counts = df[category + ' freq']
 		x_data_raw = transform(not_counts, df.index, counts)
 		y_data_raw = transform(counts, df.index, not_counts)
@@ -229,7 +401,7 @@ class ScatterChart:
 			return vec
 		return vec + np.random.rand(1, len(vec))[0] * self.scatterchartdata.jitter
 
-	def _term_rank_score_and_frequency_df(self, all_categories, category, scores):
+	def _term_rank_score_and_frequency_df(self, all_categories, category, other_categories, scores):
 		term_ranker = self.scatterchartdata.term_ranker(self.term_doc_matrix)
 		if self.scatterchartdata.use_non_text_features:
 			term_ranker.use_non_text_features()
@@ -237,28 +409,43 @@ class ScatterChart:
 		if self.x_coords is not None:
 			df['x'] = self.x_coords
 			df['y'] = self.y_coords
+
+		if not self.original_x is None:
+			try:
+				df['ox'] = self.original_x.values
+			except AttributeError:
+				df['ox'] = self.original_x
+
+		if not self.original_y is None:
+			try:
+				df['oy'] = self.original_y.values
+			except AttributeError:
+				df['oy'] = self.original_y
+
 		if scores is None:
-			scores = self._get_default_scores(category, df)
+			scores = self._get_default_scores(category, other_categories, df)
 		# np.array(self.term_doc_matrix.get_rudder_scores(category))
 		# df['category score'] = np.array(self.term_doc_matrix.get_rudder_scores(category))
 		category_column_name = category + ' freq'
 		df['category score'] = CornerScore.get_scores_for_category(
 			df[category_column_name],
-			self._get_not_category_term_frequency(category_column_name, df)
+			df[[c + ' freq' for c in other_categories]].sum(axis=1)
 		)
 		if self.scatterchartdata.term_significance is not None:
-			df['p'] = get_p_vals(self.term_doc_matrix, category_column_name,
+			df['p'] = get_p_vals(df, category_column_name,
 			                     self.scatterchartdata.term_significance)
 		df['not category score'] = CornerScore.get_scores_for_category(
-			self._get_not_category_term_frequency(category_column_name, df),
+			df[[c + ' freq' for c in other_categories]].sum(axis=1),
 			df[category_column_name]
 		)
 		df['color_scores'] = scores
-		df = self._filter_bigrams_by_minimum_not_category_term_freq(category_column_name, df)
-		df = filter_bigrams_by_pmis(
-			self._filter_by_minimum_term_frequency(all_categories, df),
-			threshold_coef=self.scatterchartdata.pmi_threshold_coefficient
-		)
+		if self.scatterchartdata.terms_to_include is None:
+			df = self._filter_bigrams_by_minimum_not_category_term_freq(
+				category_column_name, other_categories, df)
+			df = filter_bigrams_by_pmis(
+				self._filter_by_minimum_term_frequency(all_categories, df),
+				threshold_coef=self.scatterchartdata.pmi_threshold_coefficient
+			)
 
 		if self.scatterchartdata.filter_unigrams:
 			df = filter_out_unigrams_that_only_occur_in_one_bigram(df)
@@ -272,32 +459,36 @@ class ScatterChart:
 		df = df.reset_index()
 		return df
 
-	def _filter_bigrams_by_minimum_not_category_term_freq(self, category_column_name, df):
-		return df[(df[category_column_name] > 0)
-		          | (self._get_not_category_term_frequency(category_column_name, df)
-		             >= self.scatterchartdata.minimum_not_category_term_frequency)]
+	def _filter_bigrams_by_minimum_not_category_term_freq(self, category_column_name, other_categories, df):
+		if self.scatterchartdata.terms_to_include is None:
+			return df[(df[category_column_name] > 0)
+			          | (df[[c + ' freq' for c in other_categories]].sum(axis=1)
+			             >= self.scatterchartdata.minimum_not_category_term_frequency)]
+		else:
+			return df
 
 	def _filter_by_minimum_term_frequency(self, all_categories, df):
-		return df[df[all_categories].sum(axis=1)
-		          > self.scatterchartdata.minimum_term_frequency]
+		if self.scatterchartdata.terms_to_include is None:
+			return df[df[[c + ' freq' for c in all_categories]].sum(axis=1)
+			          > self.scatterchartdata.minimum_term_frequency]
+		else:
+			return df
 
 	def _limit_max_terms(self, category, df):
 		df['score'] = self._term_importance_ranks(category, df)
 		df = df.ix[df.sort_values('score').iloc[:self.scatterchartdata.max_terms].index]
 		return df[[c for c in df.columns if c != 'score']]
 
-	def _get_default_scores(self, category, df):
+	def _get_default_scores(self, category, other_categories, df):
 		category_column_name = category + ' freq'
 		cat_word_counts = df[category_column_name]
-		not_cat_word_counts = self._get_not_category_term_frequency(category_column_name, df)
+		not_cat_word_counts = df[[c + ' freq' for c in other_categories]].sum(axis=1)
 		scores = ScaledFScore.get_scores(cat_word_counts, not_cat_word_counts)
 		return scores
 
 	def _term_importance_ranks(self, category, df):
 		return np.array([df['category score rank'], df['not category score rank']]).min(axis=0)
 
-	def _get_not_category_term_frequency(self, category_column_name, df):
-		return df[[c for c in df.columns if c != category_column_name]].sum(axis=1)
 
 	def draw(self,
 	         category,
@@ -328,7 +519,7 @@ class ScatterChart:
 		except:
 			raise Exception("mpld3 need to be installed to use this function.")
 		all_categories, other_categories = self._get_category_names(category)
-		df = self._term_rank_score_and_frequency_df(all_categories, category, scores)
+		df = self._term_rank_score_and_frequency_df(all_categories, category, other_categories, scores)
 		if self.x_coords is None:
 			df['x'], df['y'] = self._get_coordinates_from_transform_and_jitter_frequencies \
 				(category, df, other_categories, transform)
