@@ -18,6 +18,7 @@ from scattertext.CSRMatrixTools import delete_columns
 from scattertext.Common import DEFAULT_BETA, DEFAULT_SCALER_ALGO, DEFAULT_BACKGROUND_SCALER_ALGO, \
 	DEFAULT_BACKGROUND_BETA
 from scattertext.FeatureOuput import FeatureLister
+from scattertext.termranking import AbsoluteFrequencyRanker
 from scattertext.termscoring.CornerScore import CornerScore
 
 warnings.simplefilter(action="ignore", category=SettingWithCopyWarning)
@@ -29,6 +30,11 @@ SPACY_ENTITY_TAGS = ['person', 'norp', 'facility', 'org', 'gpe',
                      'type', 'date', 'time', 'percent', 'money', 'quantity',
                      'ordinal', 'cardinal']
 
+MY_ENGLISH_STOP_WORDS = set(ENGLISH_STOP_WORDS) |  {'hasn', 'won', 'don', 'haven', 'shouldn', 'isn', 'couldn', 'wouldn', 'aren', 'didn', 'wasn', 'dosen', 'weren', 'doesn'}
+
+class CannotCreateATermDocMatrixWithASignleCategoryException(Exception):
+	pass
+
 
 class TermDocMatrix(object):
 	'''
@@ -60,12 +66,36 @@ class TermDocMatrix(object):
 		unigram_frequency_path : str or None
 			Path to term frequency file.
 		'''
+		if all(y == y[0]):
+			raise CannotCreateATermDocMatrixWithASignleCategoryException(
+				'Documents must be labeled with more than one category. All documents were labeled '
+				'with category: "' + str(category_idx_store.getval(y[0])) + '"')
 		self._X, self._mX, self._y, self._term_idx_store, self._category_idx_store = \
 			X, mX, y, term_idx_store, category_idx_store
 		self._metadata_idx_store = metadata_idx_store
 		self._unigram_frequency_path = unigram_frequency_path
 		self._background_corpus = None
 
+	def get_default_stoplist(self):
+		return MY_ENGLISH_STOP_WORDS
+
+	def compact(self, compactor):
+		'''
+		Compact term document matrix.
+
+		Parameters
+		----------
+		compactor : object
+			Object that takes a Term Doc Matrix as its first argument, and has a compact function which returns a
+			Term Doc Matrix like argument
+
+		Returns
+		-------
+		New Term Doc Matrix
+		'''
+		return compactor.compact(self)
+
+
 	def get_num_terms(self):
 		'''
 		Returns
@@ -118,15 +148,32 @@ class TermDocMatrix(object):
 		newX = csr_matrix((self._X.data, (row, self._X.indices)))
 		return self._term_freq_df_from_matrix(newX)
 
+	def get_term_doc_count_df(self):
+		'''
+
+		Returns
+		-------
+		pd.DataFrame indexed on terms, with columns the number of documents each term appeared in
+		'''
+		row = self._row_category_ids()
+		newX = csr_matrix(((self._X.data > 0).astype(int), (row, self._X.indices)))
+		return self._term_freq_df_from_matrix(newX)
+
 	def _term_freq_df_from_matrix(self, catX):
 		return self._get_freq_df_using_idx_store(catX, self._term_idx_store)
 
 	def _get_freq_df_using_idx_store(self, catX, idx_store):
 		d = {'term': idx_store._i2val}
 		for idx, cat in self._category_idx_store.items():
-			d[cat + ' freq'] = catX[idx, :].A[0]
+			try:
+				d[cat + ' freq'] = catX[idx, :].A[0]
+			except IndexError:
+				self._fix_problem_when_final_category_index_has_no_terms(cat, catX, d)
 		return pd.DataFrame(d).set_index('term')
 
+	def _fix_problem_when_final_category_index_has_no_terms(self, cat, catX, d):
+		d[cat + ' freq'] = np.zeros(catX.shape[1])
+
 	def get_metadata_freq_df(self):
 		'''
 		Returns
@@ -152,6 +199,14 @@ class TermDocMatrix(object):
 	def _metadata_freq_df_from_matrix(self, catX):
 		return self._get_freq_df_using_idx_store(catX, self._metadata_idx_store)
 
+	def get_terms(self):
+		'''
+		Returns
+		-------
+		np.array of unique terms
+		'''
+		return self._term_idx_store._i2val
+
 	def get_category_names_by_row(self):
 		'''
 		Returns
@@ -171,6 +226,15 @@ class TermDocMatrix(object):
 		                   if ' ' in term or "'" in term]
 		return self.remove_terms(terms_to_ignore)
 
+	def remove_infrequent_words(self, minimum_term_count, term_ranker=AbsoluteFrequencyRanker):
+		'''
+		Returns
+		-------
+		A new TermDocumentMatrix consisting of only terms which occur at least minimum_term_count.
+		'''
+		tdf = term_ranker(self).get_ranks().sum(axis=1)
+		return self.remove_terms(list(tdf[tdf <= minimum_term_count].index))
+
 	def remove_entity_tags(self):
 		'''
 		Returns
@@ -195,7 +259,7 @@ class TermDocMatrix(object):
 		A new TermDocumentMatrix consisting of only unigrams in the current TermDocumentMatrix.
 		'''
 		if stoplist is None:
-			stoplist = ENGLISH_STOP_WORDS
+			stoplist = self.get_default_stoplist()
 		else:
 			stoplist = [w.lower() for w in stoplist]
 		return self._remove_terms_from_list(stoplist)
@@ -213,7 +277,7 @@ class TermDocMatrix(object):
 		'''
 		if type(custom_stoplist) == str:
 			custom_stoplist = [custom_stoplist]
-		return self._remove_terms_from_list(set(ENGLISH_STOP_WORDS)
+		return self._remove_terms_from_list(set(self.get_default_stoplist())
 		                                    | set(w.lower() for w in custom_stoplist))
 
 	def _remove_terms_from_list(self, stoplist):
@@ -277,18 +341,123 @@ class TermDocMatrix(object):
 			idx_to_delete_list.append(self._term_idx_store.getidx(term))
 		return self.remove_terms_by_indices(idx_to_delete_list)
 
+	def keep_only_these_categories(self, categories, ignore_absences=False):
+		'''
+		Non destructive category removal.
+
+		Parameters
+		----------
+		categories : list
+			list of categories to keep
+		ignore_absences : bool, False by default
+			if categories does not appear, don't raise an error, just move on.
+
+		Returns
+		-------
+		TermDocMatrix, new object with categories removed.
+		'''
+		if not ignore_absences:
+			assert set(self.get_categories()) & set(categories) == set(categories)
+		categories_to_remove = [c for c in self.get_categories() if c not in categories]
+		return self.remove_categories(categories_to_remove)
+
+	def remove_categories(self, categories, ignore_absences=False):
+		'''
+		Non destructive category removal.
+
+		Parameters
+		----------
+		categories : list
+			list of categories to remove
+		ignore_absences : bool, False by default
+			if categories does not appear, don't raise an error, just move on.
+
+		Returns
+		-------
+		TermDocMatrix, new object with categories removed.
+		'''
+		idx_to_delete_list = []
+		existing_categories = set(self.get_categories())
+		for category in categories:
+			if category not in existing_categories:
+				if not ignore_absences:
+					raise KeyError('Category %s not found' % (category))
+				continue
+			idx_to_delete_list.append(self._category_idx_store.getidx(category))
+		new_category_idx_store = self._category_idx_store.batch_delete_idx(idx_to_delete_list)
+
+		columns_to_delete = np.nonzero(np.isin(self._y, idx_to_delete_list))
+		new_X = delete_columns(self._X.T, columns_to_delete).T
+		new_mX = delete_columns(self._mX.T, columns_to_delete).T
+		intermediate_y = self._y[~np.isin(self._y, idx_to_delete_list)]
+		old_y_to_new_y = [self._category_idx_store.getidx(x)
+		                  for x in new_category_idx_store._i2val]
+		new_y = np.array([old_y_to_new_y.index(i) if i in old_y_to_new_y else None
+		                  for i in range(intermediate_y.max() + 1)])[intermediate_y]
+
+		new_metadata_idx_store = self._metadata_idx_store
+		if len(self._metadata_idx_store):
+			meta_idx_to_delete = np.nonzero(new_mX.sum(axis=0).A1 == 0)[0]
+			new_metadata_idx_store = self._metadata_idx_store.batch_delete_idx(meta_idx_to_delete)
+
+		term_idx_to_delete = np.nonzero(new_X.sum(axis=0).A1 == 0)[0]
+		new_term_idx_store = self._term_idx_store.batch_delete_idx(term_idx_to_delete)
+		new_X = delete_columns(new_X, term_idx_to_delete)
+
+		term_doc_mat_to_ret = self._make_new_term_doc_matrix(new_X, new_mX, new_y, new_term_idx_store,
+		                                                     new_category_idx_store,
+		                                                     new_metadata_idx_store, ~np.isin(self._y, idx_to_delete_list))
+		return term_doc_mat_to_ret
+
 	def remove_terms_by_indices(self, idx_to_delete_list):
+		'''
+		Parameters
+		----------
+		idx_to_delete_list, list
+
+		Returns
+		-------
+		TermDocMatrix
+		'''
 		new_term_idx_store = self._term_idx_store.batch_delete_idx(idx_to_delete_list)
 		new_X = delete_columns(self._X, idx_to_delete_list)
-		return self._term_doc_matrix_with_new_X(new_X, new_term_idx_store)
+		return self._make_new_term_doc_matrix(new_X,
+		                                      self._mX,
+		                                      self._y,
+		                                      new_term_idx_store,
+		                                      self._category_idx_store,
+		                                      self._metadata_idx_store,
+		                                      self._y == self._y)
 
-	def _term_doc_matrix_with_new_X(self, new_X, new_term_idx_store):
+	def remove_terms_used_in_less_than_num_docs(self, threshold):
+		'''
+		Parameters
+		----------
+		threshold: int
+			Minimum number of documents term should appear in to be kept
+
+		Returns
+		-------
+		TermDocMatrix, new object with terms removed.
+		'''
+		term_counts = self._X.astype(bool).astype(int).sum(axis=0).A[0]
+		terms_to_remove = np.where(term_counts < threshold)[0]
+		return self.remove_terms_by_indices(terms_to_remove)
+
+	def _make_new_term_doc_matrix(self,
+	                              new_X,
+	                              new_mX,
+	                              new_y,
+	                              new_term_idx_store,
+	                              new_category_idx_store,
+	                              new_metadata_idx_store,
+	                              new_y_mask):
 		return TermDocMatrix(X=new_X,
-		                     mX=self._mX,
-		                     y=self._y,
+		                     mX=new_mX,
+		                     y=new_y,
 		                     term_idx_store=new_term_idx_store,
-		                     category_idx_store=self._category_idx_store,
-		                     metadata_idx_store=self._metadata_idx_store,
+		                     category_idx_store=new_category_idx_store,
+		                     metadata_idx_store=new_metadata_idx_store,
 		                     unigram_frequency_path=self._unigram_frequency_path)
 
 	def get_posterior_mean_ratio_scores(self, category):
@@ -654,10 +823,10 @@ class TermDocMatrix(object):
 			unigram_freq_table_buf = StringIO(pkgutil.get_data('scattertext', 'data/count_1w.txt')
 			                                  .decode('utf-8'))
 		to_ret = (pd.read_table(unigram_freq_table_buf,
-		                       names=['word', 'background'])
-		         .sort_values(ascending=False, by='background')
-		         .drop_duplicates(['word'])
-		         .set_index('word'))
+		                        names=['word', 'background'])
+		          .sort_values(ascending=False, by='background')
+		          .drop_duplicates(['word'])
+		          .set_index('word'))
 		return to_ret
 
 	def list_extra_features(self):
@@ -670,7 +839,6 @@ class TermDocMatrix(object):
 		                     self._metadata_idx_store,
 		                     self.get_num_docs()).output()
 
-
 	def get_scaled_f_scores_vs_background(self,
 	                                      scaler_algo=DEFAULT_BACKGROUND_SCALER_ALGO,
 	                                      beta=DEFAULT_BACKGROUND_BETA):
