@@ -8,8 +8,6 @@ from scattertext.representations.Word2VecFromParsedCorpus import GensimPhraseAdd
 from scattertext.test.test_corpusFromPandas import get_docs_categories
 
 
-
-
 class TestGensimPhraseAdder(TestCase):
 	@classmethod
 	def setUp(cls):
@@ -25,4 +23,8 @@ class TestGensimPhraseAdder(TestCase):
 		cls.corpus = CorpusFromParsedDocuments(cls.df, 'category', 'parsed').build()
 
 	def test_add_phrase(self):
-		res = GensimPhraseAdder().add_phrases(self.corpus)
\ No newline at end of file
+		adder = GensimPhraseAdder()
+		# to do
+		#res = adder.add_phrases(self.corpus)
+
+		# self.fail()
